//
//  ThreadTargetTest.cpp
//
//  Created  by Setsu on 7/16/2010.
//  Modified by Setsu on May 30, 2013.
//  Copyright 2010-2013 RoundSquare Inc. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

//----------------------------------------
//	include
//----------------------------------------
#include <Poco/Runnable.h>
#include <Poco/Random.h>
#include <Poco/Thread.h>
#include <Poco/Format.h>
#include <Poco/ThreadPool.h>
#include <Poco/ThreadTarget.h>
#include <Poco/RunnableAdapter.h>

#include <string>

#include "ScopedLogMessage.h"
#include "PrepareConsoleLogger.h"

//----------------------------------------
//	const
//----------------------------------------
const long kSleepStartInterval	= 30;
const long kSleepMemberMethod	= 300;
const long kSleepStaticMethod	= 500;
const long kSleepFunction		= 400;

//----------------------------------------
//	global
//----------------------------------------
ScopedLogMessage* g_pMsg = NULL;

//----------------------------------------
//	TestThreadTarget
//----------------------------------------
class TestThreadTarget
{
public:
	TestThreadTarget(ScopedLogMessage& msg) :
		m_Msg(msg)
	{
	}

	void MemberMethod(void)
	{
		m_Msg.Message("  MemberMethod() start");
		Poco::Thread::sleep(kSleepMemberMethod);
		m_Msg.Message("  MemberMethod() end");
	}

	static void StaticMethod(void);

private:
	ScopedLogMessage&	m_Msg;
};

void TestThreadTarget::StaticMethod(void)
{
	if(NULL != g_pMsg)
	{
		g_pMsg->Message("  StaticMethod() start");
		Poco::Thread::sleep(kSleepStaticMethod);
		g_pMsg->Message("  StaticMethod() end");
	}
}

//----------------------------------------
//	TestFunction
//----------------------------------------
void TestFunction(void)
{
	if(NULL != g_pMsg)
	{
		g_pMsg->Message("  TestFunction() start");
		Poco::Thread::sleep(kSleepFunction);
		g_pMsg->Message("  TestFunction() end");
	}
}

//----------------------------------------
//	main
//----------------------------------------
int main(int /*argc*/, char** /*argv*/)
{
	PrepareConsoleLogger logger(Poco::Logger::ROOT, Poco::Message::PRIO_INFORMATION);

	ScopedLogMessage msg("ThreadTargetTest ", "start", "end");
	g_pMsg = &msg;

	TestThreadTarget target(msg);

	Poco::ThreadTarget raStatic(&TestThreadTarget::StaticMethod);
	Poco::ThreadTarget raFunction(TestFunction);
	Poco::RunnableAdapter<TestThreadTarget> raMember(target, &TestThreadTarget::MemberMethod);

	Poco::ThreadPool::defaultPool().start(raStatic);
	Poco::Thread::sleep(kSleepStartInterval);
	Poco::ThreadPool::defaultPool().start(raFunction);
	Poco::Thread::sleep(kSleepStartInterval);
	Poco::ThreadPool::defaultPool().start(raMember);

	Poco::ThreadPool::defaultPool().joinAll();

	return 0;
}
